;;; navi2ch-article.el --- article view module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'cl)
(require 'navi2ch-message)
(require 'browse-url)

(defvar navi2ch-article-buffer-name "*navi2ch article*")
(defvar navi2ch-article-current-article nil)
(defvar navi2ch-article-current-board nil)
(defvar navi2ch-article-article-list nil)
(defvar navi2ch-article-enable-diff t
  "$B:9J,$r<h$C$F$/$k$+$I$&$+!#(Bnil $B$J$i>o$K:9J,$r<h$C$F$3$J$$(B")
(defvar navi2ch-article-enable-fill nil
  "fill-region $B$9$k$+$I$&$+(B")

(defvar navi2ch-article-replace-regexp-alist
  '(("&gt;" . ">")
    ("&lt;" . "<")
    ("&quot;" . "\"")
    ("&amp" . "&")
    ("$B!w!.(B" . ",")))

(defvar navi2ch-article-replace-regexp
  (regexp-opt (mapcar
               'car
               navi2ch-article-replace-regexp-alist)))

(defvar navi2ch-article-mode-map nil)
(unless navi2ch-article-mode-map
  (setq navi2ch-article-mode-map (make-sparse-keymap))
  (define-key navi2ch-article-mode-map "q" 'navi2ch-article-exit)
  (define-key navi2ch-article-mode-map " " 'navi2ch-article-scroll-up)
  (define-key navi2ch-article-mode-map "s" 'navi2ch-article-sync)
  (define-key navi2ch-article-mode-map "S" 'navi2ch-article-sync-disable-diff)
  (define-key navi2ch-article-mode-map [del] 'navi2ch-article-scroll-down)
  (define-key navi2ch-article-mode-map [backspace] 'navi2ch-article-scroll-down)
  (define-key navi2ch-article-mode-map "\177" 'navi2ch-article-scroll-down)
  (define-key navi2ch-article-mode-map "a" 'navi2ch-article-add-bookmark)
  (define-key navi2ch-article-mode-map "b" 'navi2ch-article-open-bookmark)
  (define-key navi2ch-article-mode-map "w" 'navi2ch-article-write-message)
  (define-key navi2ch-article-mode-map "o" 'navi2ch-toggle-offline)
  (define-key navi2ch-article-mode-map "\r" 'navi2ch-article-select-current-link)
  (define-key navi2ch-article-mode-map [mouse-2] 'navi2ch-article-mouse-select)
  (define-key navi2ch-article-mode-map "g" 'navi2ch-article-goto-number)
  (define-key navi2ch-article-mode-map ">" 'navi2ch-article-end-of-buffer)
  (define-key navi2ch-article-mode-map "<" 'navi2ch-article-beginning-of-buffer))

(defun navi2ch-article-scroll-up ()
  (interactive)
  (scroll-up)
  (force-mode-line-update))

(defun navi2ch-article-scroll-down ()
  (interactive)
  (scroll-down)
  (force-mode-line-update))

(defun navi2ch-article-end-of-buffer ()
  (interactive)
  (end-of-buffer)
  (force-mode-line-update))

(defun navi2ch-article-beginning-of-buffer ()
  (interactive)
  (beginning-of-buffer)
  (force-mode-line-update))

(defsubst navi2ch-article-get-url (board article)
  (concat (cdr (assq 'uri board))
          "dat/" (cdr (assq 'artid article))))

(defsubst navi2ch-article-get-file-name (board article)
  (navi2ch-expand-file-name
   (concat (cdr (assq 'id board))
           "/" (cdr (assq 'artid article)))))

(defsubst navi2ch-article-get-info-file-name (board article)
  (navi2ch-expand-file-name
   (concat (cdr (assq 'id board))
           "/info/" (cdr (assq 'artid article)))))

(defun navi2ch-article-get-article-list (file)
  "file $B$+$i%9%l$N(B list $B$r:n$k(B"
  (when (file-exists-p file)
    (let ((content (with-temp-buffer
                     (navi2ch-insert-file-contents file)
                     (buffer-string)))
          (i 1)
          article-list)
      (message "parsing current articles...")
      (setq article-list
            (mapcar
             (lambda (x)
               (let* ((strs (split-string x "\\(,\\|<> *\\)"))
                      (list (list
                             (cons 'number i)
                             (cons 'name (nth 0 strs))
                             (cons 'mail (nth 1 strs))
                             (cons 'date (nth 2 strs))
                             (cons 'data (nth 3 strs)))))
                 ;; (cons 'title (nth 4 strs))
                 (let ((case-fold-search t)
                       str)
                   (setq str (cdr (assq 'name list))
                         str (navi2ch-article-replace-string str))
                   (setcdr (assq 'name list) str)
                   (setq str (cdr (assq 'data list))
                         str (navi2ch-replace-string " <br> " "\n" str t)
                         str (navi2ch-article-replace-string str))
                   (setcdr (assq 'data list) str)
                   (setq str (cdr (assq 'mail list))
                         str (navi2ch-article-replace-string str))
                   (setcdr (assq 'mail list) str))
                 (setq i (1+ i))
                 list))
             (split-string content "\n")))
      (message "parsing current articles...done")
      article-list)))
                      
(defun navi2ch-article-replace-string (str)
  (setq str (navi2ch-replace-string "<[^<]*>" "" str t))
  (let (start)
    (while (setq start (string-match
                        navi2ch-article-replace-regexp
                        (or str "") start))
      (setq str (replace-match
                 (cdr
                  (assoc
                   (match-string 0 str)
                   navi2ch-article-replace-regexp-alist))
                 nil nil str))))
  str)

(defun navi2ch-article-insert-articles (list)
  "list $B$r@07A$7$FA^F~$9$k(B"
  (message "inserting current articles...")
  (dolist (elt list)
    (nconc elt (list (cons 'point (point-marker))))
    (let ((p (point)))
      (insert "From: [" (number-to-string
                         (cdr (assq 'number elt)))
              "] "
              (cdr (assq 'name elt)) 
              (if (string= (cdr (assq 'mail elt)) "")
                  ""
                (concat " <" (cdr (assq 'mail elt)) ">")))
      (put-text-property p (point) 'face 'underline)
      (put-text-property p (1+ p)
                         'current-number (cdr (assq 'number elt))))
    
    (insert "\n")
    (let ((p (point)))
      (insert "Date: " (cdr (assq 'date elt)))
      (put-text-property p (point) 'face 'underline))
    (insert "\n\n")
    (let ((p (point)))
      (insert (cdr (assq 'data elt)) "\n\n")
      ;; $BJL4X?t$NJ}$,$$$$$s$+$J!)(B
      (save-excursion
        (goto-char p)
        (while (re-search-forward ">>\\([0-9]+\\)" nil t)
          (add-text-properties
           (match-beginning 0)
           (match-end 0)
           (list 'face 'bold
                 'mouse-face 'highlight
                 'number (string-to-number
                          (match-string 1)))))
        (goto-char p)
        (while (re-search-forward "http://[^ \n]+" nil t)
          (add-text-properties
           (match-beginning 0)
           (match-end 0)
           (list 'face 'bold
                 'mouse-face 'highlight
                 'url (match-string 0)))))
      (when navi2ch-article-enable-fill
        (set-hard-newline-properties p (point))
        (let ((fill-column (- (window-width) 5))
              (use-hard-newlines t))
          (fill-region p (point)))))
    (insert (make-string (1- (window-width)) ?-) "\n"))
  (message "inserting current articles...done"))

(defun navi2ch-article-view-article (board article &optional force)
  "$B%9%l$r8+$k!#(Bforce $B$G6/@)FI$_9~$_(B"
  (switch-to-buffer (get-buffer-create navi2ch-article-buffer-name))
  (setq mode-line-buffer-identification
        (cdr (assq 'subject article)))
  (navi2ch-article-mode)
  (let ((buffer-read-only nil))
    (save-excursion
      (if (eq (assq 'artid navi2ch-article-current-article)
              (assq 'artid article))
          (navi2ch-article-sync force)
        (setq navi2ch-article-current-board board)
        (setq navi2ch-article-current-article
              (append article
                      (navi2ch-load-info
                       (navi2ch-article-get-info-file-name board article))))
        (navi2ch-article-sync force t)))
    (navi2ch-article-open-bookmark)))


(defun navi2ch-article-mode ()
"\\{navi2ch-article-mode-map}"
  (interactive)
  (setq major-mode 'navi2ch-article-mode)
  (setq mode-name "Navi2ch Article")
  (setq buffer-read-only t)
  (make-local-variable 'truncate-partial-width-windows)
  (setq truncate-partial-width-windows nil)
  (use-local-map navi2ch-article-mode-map)
  (run-hooks 'navi2ch-article-mode-hook)
  (force-mode-line-update))

(defun navi2ch-article-exit ()
  (interactive)
  (navi2ch-article-add-bookmark)
  (navi2ch-save-info
   (navi2ch-article-get-info-file-name
    navi2ch-article-current-board
    navi2ch-article-current-article)
   (list
    (assq 'bookmark navi2ch-article-current-article)))
  
  (delete-windows-on (get-buffer navi2ch-article-buffer-name))
  (bury-buffer navi2ch-article-buffer-name)
  (let ((win (get-buffer-window navi2ch-board-buffer-name)))
    (if win
        (select-window win)
      (switch-to-buffer navi2ch-board-buffer-name))))

(defun navi2ch-article-sync (&optional force first)
  "$B%9%l$r99?7$9$k!#(Bforce $B$J$i6/@)!#(B
first $B$,(B nil $B$J$i$P!"%U%!%$%k$,99?7$5$l$F$J$1$l$P2?$b$7$J$$(B"
  (interactive "P")
  (save-excursion
    (let ((buffer-read-only nil)
          (navi2ch-net-force-update (or navi2ch-net-force-update
                                        force))
          (url (navi2ch-article-get-url
                navi2ch-article-current-board
                navi2ch-article-current-article))
          (file (navi2ch-article-get-file-name
                 navi2ch-article-current-board
                 navi2ch-article-current-article))
          updated)
      (unless navi2ch-offline
        (setq updated (navi2ch-net-update-file
                       url file navi2ch-article-enable-diff)))
      (when (or first updated)
        (erase-buffer)
        (setq navi2ch-article-article-list
              (navi2ch-article-get-article-list file))
        (navi2ch-article-insert-articles navi2ch-article-article-list)))))

(defun navi2ch-article-sync-disable-diff (&optional force)
  (interactive "P")
  (let ((navi2ch-article-enable-diff nil))
    (navi2ch-article-sync force)))

(defun navi2ch-article-add-bookmark ()
  (interactive)
  (let ((elt (assq 'bookmark navi2ch-article-current-article)))
    (if elt
        (setcdr elt (navi2ch-article-get-current-number))
      (setq navi2ch-article-current-article
            (cons (cons 'bookmark (navi2ch-article-get-current-number))
                  navi2ch-article-current-article)))))

(defun navi2ch-article-open-bookmark ()
  (interactive)
  (let ((elt (cdr (assq 'bookmark navi2ch-article-current-article))))
    (when elt
      (navi2ch-article-goto-number elt))))

(defun navi2ch-article-write-message ()
  (interactive)
  (navi2ch-message-write-message navi2ch-board-current-board
                                 navi2ch-article-current-article))

(defun navi2ch-article-select-current-link ()
  (interactive)
  (let ((prop (get-text-property (point) 'number)))
    (if prop
        (navi2ch-article-goto-number prop)
      (setq prop (get-text-property (point) 'url))
      (browse-url prop))))

(defun navi2ch-article-mouse-select (e)
  (interactive "e")
  (mouse-set-point e)
  (navi2ch-article-select-current-link))

(defun navi2ch-article-goto-number (num)
  (interactive "ninput number: ")
  (push-mark)
  (goto-char
   (cdr (assq 'point (navi2ch-article-get-message num)))))

(defun navi2ch-article-get-message (num)
  "num $BHVL\$N%l%9$rF@$k(B"
  (cdr (assq num
             (mapcar
              (lambda (x)
                (cons (cdr (assq 'number x)) x))
              navi2ch-article-article-list))))
             
(defun navi2ch-article-get-current-number ()
  "$B:#$N0LCV$N%l%9$NHV9f$rF@$k(B"
  (save-excursion
    (set-buffer navi2ch-article-buffer-name)
    (or (get-text-property (point) 'current-number)
        (let ((pos (previous-single-property-change
                    (point)
                    'current-number)))
          (or (get-text-property pos 'current-number)
              (get-text-property (1- pos) 'current-number))))))

(provide 'navi2ch-article)

;;; navi2ch-article.el ends here