;;; navi2ch-board.el --- subject list module for navi2ch

;; Copyright (C) 2000 by 2$B$A$c$s$M$k(B

;; Author: (not 1)
;; Keywords: network, 2ch

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:

(require 'navi2ch-article)
(require 'cl)

(defvar navi2ch-board-buffer-name "*navi2ch board*")
(defvar navi2ch-board-window-height 10)
(defvar navi2ch-board-current-board nil)
(defvar navi2ch-board-bookmark-mode nil)
(defvar navi2ch-board-subject-list nil)

(defvar navi2ch-board-mode-map nil)
(unless navi2ch-board-mode-map
  (setq navi2ch-board-mode-map (make-sparse-keymap))
  (define-key navi2ch-board-mode-map "\r" 'navi2ch-board-select-article)
  (define-key navi2ch-board-mode-map "q" 'navi2ch-board-exit)
  (define-key navi2ch-board-mode-map "s" 'navi2ch-board-sync)
  (define-key navi2ch-board-mode-map " " 'navi2ch-board-select-article)
  (define-key navi2ch-board-mode-map [del] 'scroll-down)
  (define-key navi2ch-board-mode-map [backspace] 'scroll-down)
  (define-key navi2ch-board-mode-map "n" 'next-line)
  (define-key navi2ch-board-mode-map "p" 'previous-line)
  (define-key navi2ch-board-mode-map "a" 'navi2ch-board-add-bookmark)
  (define-key navi2ch-board-mode-map "d" 'navi2ch-board-delete-bookmark)
  (define-key navi2ch-board-mode-map "b" 'navi2ch-board-toggle-bookmark)
  (define-key navi2ch-board-mode-map "o" 'navi2ch-toggle-offline)
  (define-key navi2ch-board-mode-map [mouse-2] 'navi2ch-board-mouse-select)
  (define-key navi2ch-board-mode-map "g" 'navi2ch-list-goto-board)
  (define-key navi2ch-board-mode-map ">" 'end-of-buffer)
  (define-key navi2ch-board-mode-map "<" 'beginning-of-buffer))

(defsubst navi2ch-board-get-url (board)
  (concat (cdr (assq 'uri board)) "subject.txt"))

(defsubst navi2ch-board-get-file-name (board &optional file-name)
  (navi2ch-expand-file-name
   (concat (cdr (assq 'id board)) (or file-name "/subject.txt"))))

(defun navi2ch-board-get-subject-list (file)
  "file $B$+$i%9%l$N(B list $B$r:n$k!#(B"
  (when (file-exists-p file)
    (with-temp-buffer
      (navi2ch-insert-file-contents file)
      (goto-char (point-min))
      (let (list)
        (while (re-search-forward
                "\\([0-9]+\\.dat\\)\\(,\\|<>\\)\\(.+\\)\n" nil t)
          (let ((id (match-string 1))
                (str (match-string 3)))
            (setq str (navi2ch-replace-string
                       " *\\([(<].+[>)]\\)$" " \\1" str))
            (setq str (navi2ch-replace-string "^ +" "" str))
            (setq list (cons (list (cons 'subject str)
                                   (cons 'artid id))
                             list))))
        (nreverse list)))))

(defun navi2ch-board-insert-subjects (list)
  (let ((prev (point)))
    (dolist (elt list)
      (insert (cdr (assq 'subject elt)) "\n")
      (put-text-property prev (point) 'article elt)
      (put-text-property prev (1- (point)) 'mouse-face 'highlight)
      (setq prev (point)))))

(defun navi2ch-board-goto-board (board &optional force)
  (switch-to-buffer (get-buffer-create navi2ch-board-buffer-name))
  (navi2ch-board-mode)
  (setq mode-line-buffer-identification
        (cdr (assq 'name board)))
  (setq navi2ch-board-bookmark-mode nil)
  (save-excursion
    (if (eq navi2ch-board-current-board board)
        (navi2ch-board-sync force)
      (setq navi2ch-board-current-board board)
      (navi2ch-board-sync force t))))

(defun navi2ch-board-mode ()
  "\\{navi2ch-board-mode-map}"
  (interactive)
  (setq major-mode 'navi2ch-board-mode)
  (setq mode-name "Navi2ch Board")
  (setq buffer-read-only t)
  (use-local-map navi2ch-board-mode-map)
  (run-hooks 'navi2ch-board-mode-hook)
  (force-mode-line-update))

(defun navi2ch-board-select-article (&optional force)
  (interactive "P")
  (let ((article (get-text-property (point) 'article)))
    (if article
        (progn
          (delete-windows-on (get-buffer navi2ch-article-buffer-name))
          (split-window-vertically navi2ch-board-window-height)
          (other-window 1)
          (navi2ch-article-view-article
           navi2ch-board-current-board article force))
      (message ("can't select this line!")))))

(defun navi2ch-board-exit ()
  (interactive)
  (navi2ch-article-exit)
  (delete-windows-on (get-buffer navi2ch-board-buffer-name))
  (bury-buffer navi2ch-board-buffer-name)
  (let ((win (get-buffer-window navi2ch-list-buffer-name)))
    (if win
        (select-window win)
      (switch-to-buffer navi2ch-list-buffer-name))))

(defun navi2ch-board-sync (&optional force first)
  (interactive "P")
  (setq navi2ch-board-bookmark-mode nil)
  (save-excursion
    (let ((buffer-read-only nil)
          (navi2ch-net-force-update (or navi2ch-net-force-update
                                        force))
          (url (navi2ch-board-get-url navi2ch-board-current-board))
          (file (navi2ch-board-get-file-name navi2ch-board-current-board))
          updated)
      (unless navi2ch-offline
        (setq updated (navi2ch-net-update-file
                       url file)))
      (when (or first updated)
        (erase-buffer)
        (setq navi2ch-board-subject-list
              (navi2ch-board-get-subject-list file))
        (navi2ch-board-insert-subjects navi2ch-board-subject-list)))))

(defun navi2ch-board-add-bookmark ()
  (interactive)
  (unless navi2ch-board-bookmark-mode
    (let ((article (get-text-property (point) 'article)))
      (if article
	  (let ((subject (cdr (assq 'subject article)))
		(artid (cdr (assq 'artid article)))
		(file-name (navi2ch-board-get-file-name "bookmark.txt")))
            (with-temp-file file-name
              (when (file-exists-p file-name)
                (navi2ch-insert-file-contents file-name)
                (goto-char (point-max)))
              (insert artid "," subject "\n")))
        (message "can't select this line!")))))

(defun navi2ch-board-delete-bookmark ()
  (interactive)
  (if (and navi2ch-board-bookmark-mode
	   (y-or-n-p "Delete this bookmark?"))
      (let ((article (get-text-property (point) 'article)))
	(if article
            (progn
              (setq navi2ch-board-subject-list
                    (delq article navi2ch-board-subject-list))
              (with-temp-file 
                  (navi2ch-board-get-file-name "bookmark.txt")
                (dolist (x navi2ch-board-subject-list)
                  (insert (cdr (assq 'artid x)) ","
                          (cdr (assq 'subject x)) "\n")))
              (let ((buffer-read-only nil))
                (delete-region (save-excursion (beginning-of-line) (point))
                               (save-excursion (forward-line) (point)))))
          (message "can't select this line!")))))

(defun navi2ch-board-toggle-bookmark ()
  (interactive)
  (setq navi2ch-board-bookmark-mode (not navi2ch-board-bookmark-mode))
  (let ((buffer-read-only nil))
    (save-excursion
      (erase-buffer)
      (setq navi2ch-board-subject-list
            (navi2ch-board-get-subject-list
             (navi2ch-board-get-file-name
              (and navi2ch-board-bookmark-mode
                   (file-exists-p (navi2ch-board-get-file-name "bookmark.txt"))
                   "bookmark.txt"))))
      (navi2ch-board-insert-subjects navi2ch-board-subject-list))))

(defun navi2ch-board-mouse-select (e)
  (interactive "e")
    (mouse-set-point e)
  (beginning-of-line)
  (save-excursion
    (navi2ch-board-select-article)))

(provide 'navi2ch-board)

;;; navi2ch-board.el ends here